# unix detection script
# copyright by Stefan Roettger
# contact: snroettg at gmail

# platform detection
platform="unknown"
description="unknown"
distributor="unknown"
version=""
# general platform checks
[ "`uname`" = "Linux" ] && platform="linux"
[ "`uname`" = "Darwin" ] && platform="macosx"
# check for linux standard base
if [ -x "$(which lsb_release 2>/dev/null)" ]; then
   description=`lsb_release -d |& sed -e 's/.*:[[:space:]]*//g'`
   distributor=`lsb_release -i |& sed -e 's/.*:[[:space:]]*//g' | tr '[a-z]' '[A-Z]'`
   release=`lsb_release -r |& sed -e 's/.*:[[:space:]]*//g'`
   [ "$distributor" = "DEBIAN" ] && platform="debian" && version=$release
   [ "$distributor" = "UBUNTU" ] && platform="ubuntu" && version=$release
   [ "$distributor" = "LINUXMINT" ] && platform="mint" && version=$release
   [ "$distributor" = "OPENSUSE" ] && platform="opensuse" && version=$release
   [ "$distributor" = "OPENSUSE PROJECT" ] && platform="opensuse" && version=$release
   [ "$distributor" = "SUSE LINUX" ] && platform="suse" && version=$release
   [ "$distributor" = "FEDORA" ] && platform="fedora" && version=$release
   [ "$distributor" = "REDHAT" ] && platform="redhat" && version=$release
# check for os-release
elif [ -e /etc/os-release 2>/dev/null ]; then
   source /etc/os-release
   description=$PRETTY_NAME
   distributor=$NAME
   release=$VERSION_ID
   platform=$ID
   version=$release
   [ "$platform" = "centos" ] && platform="fedora"
   [ "$platform" = "ol" ] && platform="redhat"
# check for maxosx
elif [ -x "$(which sw_vers 2>/dev/null)" ]; then
   macosx=`sw_vers -productVersion`
   platform=macosx
   version=$macosx
   description="MacOS $version"
   distributor="Apple"
fi
# determine major version
majorversion=`echo $version | sed -e 's/\.[0-9]*$//g'`
if [ ${majorversion//./} = $majorversion ]; then
   majorversion=$version
fi
majorversion=${majorversion/./}
version=${version//./}
[ "$version" = "" ] && version="0"

# platform checks
type="unknown"
installer="echo"
installerquiet="echo"
installerquery="false"
installopts=""
updateopts=""
if [ $platform = "debian" ] || [ $platform = "ubuntu" ] || [ $platform = "mint" ]; then
   type="debian"
   installer="apt-get"
   installerquiet="apt-get --quiet=2"
   installerquery="dpkg-query -s"
   installopts="--yes"
elif [ $platform = "opensuse" ] || [ $platform = "suse" ]; then
   type="opensuse"
   installer="zypper"
   installerquiet="zypper -q"
   installerquery="rpm -q"
   installopts="-y"
elif [ $platform = "fedora" ] || [ $platform = "redhat" ]; then
   type="fedora"
   installer="yum"
   installerquiet="yum -q"
   installerquery="rpm -q"
   installopts="-y"
elif [ $platform = "macosx" ]; then
   type="macosx"
fi

echo detected platform: $description
