#!/bin/bash
# unix online installer script
# copyright by Stefan Roettger
# contact: snroettg at gmail

# internet check
echo detecting internet connection
ping="ping -c3"
$ping www.nsa.gov >&/dev/null
if [ $? != 0 ]; then
   echo NO INTERNET CONNECTION
   exit 1
else
   echo internet connection is ok
fi

# platform check
echo detecting platform
source install_detection.sh
echo detected distributor: $distributor
if [ $type = "unknown" ]; then
   echo UNSUPPORTED PLATFORM
   echo PLEASE INSTALL MANUALLY
   exit 1
fi

# install subversion
failure=0
if [ $type = "debian" ] || [ $type = "opensuse" ] || [ $type = "fedora" ]; then
   if [ ! -x "$(which svn 2>/dev/null)" ]; then
      echo installing subversion
      sudo $installerquiet update $updateopts
      sudo $installer install $installopts subversion
      failure=$?
   fi
elif [ $type = "macosx" ]; then
   if [ ! -x "$(which svn 2>/dev/null)" ]; then
      failure=1
   else
      svn --version | grep -i "svn, version"
      failure=$?
   fi
   if [ $failure != 0 ]; then
      echo please install subversion manually
      echo e.g. with homebrew from https://brew.sh: brew install subversion
   fi
else
   if [ ! -x "$(which svn 2>/dev/null)" ]; then
      echo please install subversion manually
      failure=1
   fi
fi
if [ $failure != 0 ]; then
   echo UNABLE TO CONTINUE INSTALLATION WITHOUT SUBVERSION
   exit $failure
fi

# download package
echo downloading installation package
for i in {1..3}
do
   rm -rf download
   svn export --depth files http://svn.code.sf.net/p/glvertex/code/trunk download >&/dev/null
   failure=$?
   if [ $failure = 0 ]; then
      break
   fi
done

# install package
if [ -x download/install.sh ]; then
   echo downloading finished
   echo
   echo performing package installation
   cd download
   ./install.sh
   failure=$?
   if [ $failure != 0 ]; then
      echo UNABLE TO FINISH INSTALLATION
   fi
   if [ -d docs ]; then
      rm -rf ../docs
      mv docs index.html ..
   fi
   cd ..
else
   echo UNABLE TO DOWNLOAD INSTALLATION PACKAGE
fi
rm -rf download

exit $failure
