// (c) by Stefan Roettger, licensed under MIT license

#ifndef GLSLEDITOR_H
#define GLSLEDITOR_H

#include "glslwidget.h"

#include <qglobal.h>

#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
#include <QtGui>
#else
#include <QtWidgets>
#endif

#include <locale.h>

#include "glvertex_qt_codeeditor.h"

class GLSLTextEdit: public lgl_Qt_CodeEditor
{
   Q_OBJECT

public:

   //! default ctor
   GLSLTextEdit(QWidget *parent = NULL)
      : lgl_Qt_CodeEditor(parent)
   {}

   //! dtor
   virtual ~GLSLTextEdit()
   {}

   QSize minimumSizeHint() const
   {
      return(QSize(512, 256));
   }

};

class GLSLEditor: public QWidget
{
   Q_OBJECT

public:

   //! default ctor
   GLSLEditor(QWidget *parent = NULL);

   //! dtor
   virtual ~GLSLEditor();

   QSize minimumSizeHint() const
   {
      return(QSize(1024, 512));
   }

protected:

   GLSLWidget *glslwidget_;

   GLSLTextEdit *vtxshader_;
   GLSLTextEdit *frgshader_;
   float zoom_;

   QLabel *vtxstatus_;
   QLabel *frgstatus_;
   QLabel *wrnstatus_;
   QLabel *lnkstatus_;

   QString browse(QString title, QString path="", bool newfile=false);
   QString readText(QString file);

   void keyPressEvent(QKeyEvent *event);

protected slots:

   void loadObj();
   void reloadShaders();

   void vtxstatus(QString);
   void frgstatus(QString);
   void lnkstatus(QString);
   void wrnstatus(QString);

   void vtxLoad();
   void frgLoad();

   void background();
   void ztest();
   void wireframe();

   void help();
};

#endif
