// (c) by Stefan Roettger, licensed under MIT license

#include <glvertex_qt_glui.h>

#include <glvertex_nodes.h>
#include <glvertex_geometry.h>

class Qt_GLWindow: public lgl_Qt_GLUI
{
public:

   Qt_GLWindow()
      : lgl_Qt_GLUI(),
        z_(50),
        root_(NULL),
        view_(0)
   {
      setFPS(0);
      toggleFPS();

      gear2_ = new lglGear(0.5, 0.25, 2.0/z_, 0, 1, 0.25);
      gear2_->create(256, 0.1);

      double angle = gear2_->contactAngle();

      gear1_ = new lglGear(1, 0.25, 2.0/z_, angle, 1, 0.25);
      gear1_->create(256, 0.1);
   }

   virtual ~Qt_GLWindow()
   {
      if (root_)
         delete root_;

      if (gear1_)
         delete gear1_;

      if (gear2_)
         delete gear2_;
   }

protected:

   int z_;
   lglGear *gear1_, *gear2_;

   lgl_Node *root_;
   lgl_CameraNode *cam_;

   int view_;

   void initializeOpenGL()
   {
      // setup OpenGL
      glClearColor(1,1,1,1);

      // create scene graph
      double fovy = 45;
      double aspect = (double)width()/height();
      double nearp = 0.1;
      double farp = 1000;
      cam_ = new lgl_CameraNode();
      cam_->setPerspective(fovy, aspect, nearp, farp);
      cam_->setEye(vec3(0,1,4), vec3(0,1,0), vec3(0,1,0));
      root_ = new lgl_Node();
      root_->add(cam_);
      lgl_Node *node = root_->add(new lgl_LightSourceNode(vec4f(0,0,0)));
      node->add(new lgl_TranslationNode(vec3(-1.5,0,0)))->add(new lgl_RotationAnimationNode(10, vec3(0,0,1)))->add(new lgl_ColorNode(vec3(0.75,0.75,1)))->add(new lgl_GeometryNode(gear1_));
      node->add(new lgl_TranslationNode(vec3(-1.5,2,0)))->add(new lgl_RotationAnimationNode(-10, vec3(0,0,1)))->add(new lgl_RotationNode(0.5/gear1_->teeth()*360, vec3(0,0,1)))->add(new lgl_ColorNode(vec3(1,0.75,0.75)))->add(new lgl_GeometryNode(gear1_));
      node->add(new lgl_TranslationNode(vec3(0,0,0)))->add(new lgl_RotationAnimationNode(-20, vec3(0,0,1)))->add(new lgl_ColorNode(vec3(0.75,1,0.75)))->add(new lgl_GeometryNode(gear2_));
      node->add(new lgl_TranslationNode(vec3(0,2,0)))->add(new lgl_RotationAnimationNode(20, vec3(0,0,1)))->add(new lgl_RotationNode(0.5/gear2_->teeth()*360, vec3(0,0,1)))->add(new lgl_ColorNode(vec3(0.75,1,0.75)))->add(new lgl_GeometryNode(gear2_));
      node->add(new lgl_TranslationNode(vec3(1.5,0,0)))->add(new lgl_RotationAnimationNode(10, vec3(0,0,1)))->add(new lgl_RotationNode(0.5/gear1_->teeth()*360, vec3(0,0,1)))->add(new lgl_ColorNode(vec3(1,0.75,0.75)))->add(new lgl_GeometryNode(gear1_));
      node->add(new lgl_TranslationNode(vec3(1.5,2,0)))->add(new lgl_RotationAnimationNode(-10, vec3(0,0,1)))->add(new lgl_ColorNode(vec3(0.75,0.75,1)))->add(new lgl_GeometryNode(gear1_));
   }

   void renderOpenGL(double dt)
   {
      // clear frame buffer
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

      // set camera aspect
      double aspect = (double)width()/height();
      cam_->setAspect(aspect);

      // render scene graph
      root_->renderSceneGraph(dt);
   }

   void keyPressed(char key)
   {
      if (key == '\r')
         toggleView();
      else if (key == 's')
         toggleCapture("screen");
   }

   std::string getKeyInfo()
   {
      const char keys[] =
         "Enter: toggle view\n"
         "s: screen capture\n";

      return(lgl_Qt_GLUI::getKeyInfo() + "\n" + keys);
   }

   void toggleView()
   {
      view_ = (view_+1)%3;

      if (view_ == 0)
         cam_->setEye(vec3(0,1,4), vec3(0,1,0), vec3(0,1,0));
      else if (view_ == 1)
         cam_->setEye(vec3(-1.5,1.15,0.75), vec3(-1.5,1,0), vec3(0,1,0));
      else
         cam_->setEye(vec3(0,2,1.5), vec3(0,2,0), vec3(0,1,0));

      lglResetManip();
      update();
   }

   void finishOpenGL(double t)
   {
      if (t > 15)
      {
         toggleView();
         restart();
      }

      lgl_Qt_GLUI::finishOpenGL(t);
   }

   void doubleClick(double x, double y)
   {
      toggle_animation();
   }

   void interaction()
   {
      restart();
   }

};

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   Qt_GLWindow main;
   main.resize(1280, 720);
   main.show();

   return(app.exec());
}
