// (c) by Stefan Roettger, licensed under MIT license

#include <glvertex_qt_glui.h>

class Qt_GLWindow: public lgl_Qt_GLUI
{
public:

   //! default ctor
   Qt_GLWindow()
      : lgl_Qt_GLUI()
   {
      obj = NULL;
   }

   //! dtor
   virtual ~Qt_GLWindow()
   {
      if (obj) delete obj;
   }

protected:

   lglVBO *obj;

   void initializeOpenGL()
   {
      // setup OpenGL
      glClearColor(1,1,1,1);
      glEnable(GL_DEPTH_TEST);
      glDisable(GL_CULL_FACE);

      // load obj and tex file
      std::string filename = "teapot.obj";
      if (QApplication::arguments().size() > 1) filename = QCoreApplication::arguments().at(1).toStdString();
      obj = lglLoadObjWithQtTexture(filename.c_str());
   }

   void renderOpenGL(double dt)
   {
      // clear frame buffer
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

      // projection setup
      double fovy = 30;
      double aspect = (double)width()/height();
      double nearp = 0.1;
      double farp = 10;
      lglMatrixMode(LGL_PROJECTION);
      lglLoadIdentity();
      lglPerspective(fovy, aspect, nearp, farp);

      // viewing setup
      lglMatrixMode(LGL_MODELVIEW);
      lglLoadIdentity();
      lglLookAt(vec3(0,0,3), vec3(0,0,0), vec3(0,1,0));

      // define local rotated coordinate system
      static double angle=0; // rotation angle in degrees
      static const double omega=10; // rotation speed in degrees/s
      angle+=dt*omega;
      lglRotate(angle, 0,1,0);

      // lighting setup
      lglLightDirection(vec3f(0,0,1), true);
      lglLightParameters(vec3f(0), vec3f(1), vec3f(0));

      // render obj
      if (obj != NULL)
      {
         lglScale(1/obj->lglGetExtent().y);
         lglTranslate(-obj->lglGetCenter());
         obj->lglRender();
      }
   }

   void keyPressed(char key)
   {
      if (key == '\r')
      {
         lglLighting(!lglGetLighting());
         float g = lglGetLighting()?1:0;
         lglClearColor(g,g,g);
      }
   }

   std::string getKeyInfo()
   {
      const char keys[] =
         "Enter: toggle lighting\n";

      return(lgl_Qt_GLUI::getKeyInfo() + "\n" + keys);
   }

   void doubleClick(double x, double y)
   {
      keyPressed('\r');
   }

};

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   // create OpenGL window
   Qt_GLWindow main;
   main.resize(640, 480);
   main.show();

   // start event loop
   return(app.exec());
}
