// (c) by Stefan Roettger, licensed under MIT license

#include <glvertex_qt_glui.h>

#ifndef LGL_GLES
#   define PROCTEX
#endif

class lglTree: public lglVBO
{
public:

   lglTree(double angle=30)
      : lglVBO()
   {
      lglMatrixMode(LGL_PREMODEL);
      lglLoadIdentity();

      lglPushMatrix();
         cube(vec3(0.75,0.625,0.5));
         lglTranslate(0,0.5,0);
         tree(0, 1, angle);
      lglPopMatrix();

      lglMatrixMode(LGL_MODELVIEW);
   }

   void tree(int level, double size, double angle)
   {
      vec3 color(0.75,0.625,0.5);

      if (size < 0.2) color = vec3(0,1,0);
      else if (size < 0.5) color = vec3(1,0.875,0.75);

      cube(color);

      if (size>0.05 && level<100)
      {
         lglTranslate(0,0.5,0);

         lglPushMatrix();
            lglTranslate(-0.5,0,0);
            lglRotate(angle, 0,0,1);
            if (level%2) lglRotate(-10, 1,0,0);
            else lglRotate(10, 1,0,0);
            double c = cos(angle*PI/180);
            lglScale(c);
            lglTranslate(0.5,0.5,0);
            tree(level+1, size*c, angle);
         lglPopMatrix();

         lglPushMatrix();
            lglTranslate(0.5,0,0);
            lglRotate(angle-90, 0,0,1);
            if (level%2) lglRotate(10, 1,0,0);
            else lglRotate(-10, 1,0,0);
            double s = sin(angle*PI/180);
            lglScale(s);
            lglTranslate(-0.5,0.5,0);
            tree(level+1, size*s, angle);
         lglPopMatrix();
      }
   }

   void cube(vec3 color)
   {
      lglBegin(LGL_QUADS);
         lglColor(color);
         // left quad
         lglNormal(-1,0,0);
         lglVertex(-0.5,-0.5,-0.5);
         lglVertex(-0.5,-0.5,0.5);
         lglVertex(-0.5,0.5,0.5);
         lglVertex(-0.5,0.5,-0.5);
         // right quad
         lglNormal(1,0,0);
         lglVertex(0.5,-0.5,0.5);
         lglVertex(0.5,-0.5,-0.5);
         lglVertex(0.5,0.5,-0.5);
         lglVertex(0.5,0.5,0.5);
         // front quad
         lglNormal(0,0,1);
         lglVertex(-0.5,-0.5,0.5);
         lglVertex(0.5,-0.5,0.5);
         lglVertex(0.5,0.5,0.5);
         lglVertex(-0.5,0.5,0.5);
         // back quad
         lglNormal(0,0,-1);
         lglVertex(0.5,-0.5,-0.5);
         lglVertex(-0.5,-0.5,-0.5);
         lglVertex(-0.5,0.5,-0.5);
         lglVertex(0.5,0.5,-0.5);
         // bottom quad
         lglNormal(0,-1,0);
         lglVertex(-0.5,-0.5,-0.5);
         lglVertex(0.5,-0.5,-0.5);
         lglVertex(0.5,-0.5,0.5);
         lglVertex(-0.5,-0.5,0.5);
         // top quad
         lglNormal(0,1,0);
         lglVertex(-0.5,0.5,0.5);
         lglVertex(0.5,0.5,0.5);
         lglVertex(0.5,0.5,-0.5);
         lglVertex(-0.5,0.5,-0.5);
      lglEnd();
   }

};

class Qt_GLWindow: public lgl_Qt_GLUI
{
public:

   //! default ctor
   Qt_GLWindow()
      : lgl_Qt_GLUI(),
        angle_(30),
        forest_(false)
   {
      tree_ = new lglTree(angle_);
      trees_ = new lglSharedVBO(tree_);

      lglMatrixMode(LGL_PREMODEL);
      lglLoadIdentity();
      for (int i=-4; i<=4; i++)
         for (int j=-4; j<=4; j++)
         {
            lglPushMatrix();
            lglTranslate(2*i,0,2*j);
            if ((i+j)%2) lglRotate(-45, 0,1,0);
            else lglRotate(45, 0,1,0);
            if ((i&2)^(j&2)) lglRotate(180, 0,1,0);
            lglScale(3.0/tree_->lglGetExtent().length());
            lglTranslate(-tree_->lglGetCenter());
            trees_->push_back();
            lglPopMatrix();
         }
      lglMatrixMode(LGL_MODELVIEW);

      program_ = 0;
   }

   //! dtor
   virtual ~Qt_GLWindow()
   {
      delete tree_;
      delete trees_;

      lglDeleteTexture(texid_);
      lglDeleteGLSLProgram(program_);
   }

protected:

   double angle_;

   lglVBO *tree_;
   lglSharedVBO *trees_;

   GLuint texid_;
   GLuint program_;

   bool forest_;

   void initializeOpenGL()
   {
      // setup OpenGL
      glClearColor(1,1,1,1);
      glEnable(GL_DEPTH_TEST);
      glEnable(GL_CULL_FACE);

#ifdef PROCTEX
      // create perlin noise texture
      texid_ = lglCreateNoiseTexmap3D(64, 64, 64);
#endif

      // create procedural shader
      program_ = createShader();

      // enable procedural shader
      tree_->lglUseProgram(program_);
      tree_->lglUniformf("density", 0.01f);
      tree_->lglUniformfv("light", vec3f(0,0,1));
      tree_->lglUniformfv("kaIa", vec3f(0.5f));
      tree_->lglUniformfv("kdId", vec3f(0.25f));
      tree_->lglUniformfv("ksIs", vec3f(1.0f));
      tree_->lglUniformf("exponent", 500);
   }

   void renderOpenGL(double dt)
   {
      double fovy = 60;
      double aspect = (double)width()/height();
      double nearp = 0.1;
      double farp = 100;

      // clear frame buffer
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

      // setup perspective matrix
      lglProjection(fovy, aspect, nearp, farp);

      // setup view matrix
      vec3 eye = forest_? vec3(0,2,5) : vec3(0,0,2);
      vec3 at(0,0,0), up(0,1,0);
      lglView(eye, at, up);

      // define local rotated coordinate system
      static double angle=0; // rotation angle in degrees
      static const double omega=5; // rotation speed in degrees/s
      angle+=dt*omega;
      lglRotate(angle, 0,1,0);

#ifdef PROCTEX
      // enable procedural texturing
      tree_->lglSampler3D("sampler", texid_);
#endif

      // render tree objects
      if (forest_)
         trees_->lglRender();
      else
      {
         lglPushMatrixScoped();
         lglScale(3.0/tree_->lglGetExtent().length());
         lglTranslate(-tree_->lglGetCenter());
         tree_->lglRender();
      }

#ifdef PROCTEX
      // disable procedural texturing
      tree_->lglSampler3D("sampler", 0);
#endif
   }

   GLuint createShader()
   {
      static const char vertex_shader[] =
         "#version " LGL_GLSL_VERSION "\n"
         "attribute vec4 vertex_position;\n"
         "attribute vec4 vertex_color;\n"
         "attribute vec3 vertex_normal;\n"
         "uniform mat4 mv;\n"
         "uniform mat4 mvp;\n"
         "uniform mat4 mvit;\n"
         "uniform vec3 light;\n"
         "varying vec4 frag_color;\n"
         "varying vec3 frag_normal;\n"
         "varying vec3 frag_halfway;\n"
         "varying vec3 frag_position;\n"
         "vec4 fvertex() {return(mvp * vertex_position);}\n"
         "vec3 fnormal() {return(normalize(mat3(mvit) * vertex_normal));}\n"
         "vec3 fhalfway() {return(normalize(light - normalize(vec3(mv * vertex_position))));}\n"
         "void main()\n"
         "{\n"
         "   frag_color = vertex_color;\n"
         "   frag_normal = fnormal();\n"
         "   frag_halfway = fhalfway();\n"
         "   frag_position = vec3(vertex_position);\n"
         "   gl_Position = fvertex();\n"
         "}\n";

      static const char fragment_shader[] =
         "#version " LGL_GLSL_VERSION "\n"
#ifdef LGL_GLES
         "precision highp float;\n"
#endif
         "uniform float density;\n"
         "uniform vec3 light;\n"
         "uniform vec3 kaIa,kdId,ksIs;\n"
         "uniform float exponent;\n"
         "varying vec4 frag_color;\n"
         "varying vec3 frag_normal;\n"
         "varying vec3 frag_halfway;\n"
         "varying vec3 frag_position;\n"
         "vec4 flighting(vec4 color)\n"
         "{\n"
         "   vec3 n = normalize(frag_normal);\n"
         "   vec3 h = normalize(frag_halfway);\n"
         "   float diffuse = dot(light, n);\n"
         "   float specular = dot(h, n);\n"
         "   diffuse = (diffuse < 0.0f)? 0.0f : diffuse;\n"
         "   specular = pow((specular < 0.0f)? 0.0f : specular, exponent);\n"
         "   return(vec4(vec3(color) * (kaIa + kdId * diffuse) + ksIs * specular, color.a));\n"
         "}\n"
#ifdef PROCTEX
         "uniform sampler3D sampler;\n"
#endif
         "void main()\n"
         "{\n"
         "   float w = 1.0f/gl_FragCoord.w;\n"
         "   w = 1.0f-exp(-density*w*w);\n"
#ifdef PROCTEX
         "   vec4 tex = texture3D(sampler, frag_position);\n"
         "   float rad = sqrt(frag_position.x*frag_position.x+frag_position.z*frag_position.z);\n"
         "   float proc = 0.5f+0.5f*sin(70.0f*rad+10.0f*tex.x);\n"
         "   vec4 proctex = vec4(vec3(0.2f+0.8f*proc*proc*proc),1);\n"
         "   vec4 color = flighting(frag_color) * proctex;\n"
#else
         "   vec4 color = flighting(frag_color);\n"
#endif
         "   gl_FragColor = (1.0f-w)*color + w*vec4(1);\n"
         "}\n";

      GLuint program = lglCompileGLSLProgram(vertex_shader, fragment_shader);

      return(program);
   }

   void keyPressed(char key)
   {
      if (key == '\r')
      {
         forest_ = !forest_;
         tree_->lglResetManip();
         update();
      }
      else if (key == '+')
      {
         angle_ += 2;
         if (angle_ > 90) angle_ = 90;

         delete tree_;
         tree_ = new lglTree(angle_);
         trees_->lglSetVBO(tree_);
         update();
      }
      else if (key == '-')
      {
         angle_ -= 2;
         if (angle_ < 0) angle_ = 0;

         delete tree_;
         tree_ = new lglTree(angle_);
         trees_->lglSetVBO(tree_);
         update();
      }
   }

   std::string getKeyInfo()
   {
      const char keys[] =
         "Enter: toggle forest\n"
         "+: increase forking angle\n"
         "-: decrease forking angle\n";

      return(lgl_Qt_GLUI::getKeyInfo() + "\n" + keys);
   }

   void doubleClick(double x, double y)
   {
      keyPressed('\r');
   }

};

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   Qt_GLWindow main;
   main.resize(1280, 720);
   main.show();

   return(app.exec());
}
