// (c) by Stefan Roettger, licensed under MIT license

#include <glvertex_qt_glui.h>

class Qt_GLWindow: public lgl_Qt_GLUI
{
public:

   //! default ctor
   Qt_GLWindow()
      : lgl_Qt_GLUI()
   {
      teapot = NULL;
   }

   //! dtor
   virtual ~Qt_GLWindow()
   {
      if (teapot)
         delete teapot;
   }

protected:

   lglVBO *teapot;

   void initializeOpenGL()
   {
      // setup OpenGL
      glClearColor(1,1,1,1);
      glEnable(GL_DEPTH_TEST);
      glDisable(GL_CULL_FACE);

      // load teapot
      const char file1[] = "teapot.obj";
      QString file2(":/obj/teapot.obj");
      teapot = lglLoadObj(file1, true);
      if (!teapot) teapot = lglLoadObj(file2);
   }

   void renderOpenGL(double dt)
   {
      double fovy = 45;
      double aspect = (double)width()/height();
      double nearp = 0.1;
      double farp = 10;

      // clear frame buffer
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

      // setup perspective matrix
      double fovx = aspect>1?fovy:aspect>0.5?fovy/aspect:2*fovy;
      lglProjection(fovx, aspect, nearp, farp);

      // setup view matrix
      lglView(vec3(0,1,2), vec3(0,0,0), vec3(0,1,0));

      // define local rotated coordinate system
      static double angle=0; // rotation angle in degrees
      static const double omega=10; // rotation speed in degrees/s
      angle+=dt*omega;
      lglRotate(angle, 0,1,0);

      // render teapot obj
      if (teapot)
      {
         lglPushMatrixScoped();
         lglScale(2.0/teapot->lglGetExtent().length());
         lglTranslate(-teapot->lglGetCenter());
         teapot->lglRender();
      }
   }

};

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   Qt_GLWindow main;
   main.resize(640, 480);
   main.show();

   return(app.exec());
}
